SystemOrganization addCategory: #'Seaside-Pharo-Development-Core'!
SystemOrganization addCategory: #'Seaside-Pharo-Development-Core-Plugins'!
SystemOrganization addCategory: #'Seaside-Pharo-Development-Core-Profiler'!

WAObject subclass: #WADevelopment
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Pharo-Development-Core'!

!WADevelopment commentStamp: 'lr 10/18/2008 10:06' prior: 0!
A WADevelopment is a collection of convenience methods for dealing with Seaside development in Squeak. This code is not supposed to be ported.!

!WADevelopment class methodsFor: 'monticello-repositories' stamp: 'jf 8/31/2009 14:38'!
addRepository: aRepository toWorkingCopies: aCollection
	aCollection do: [ :each | each repositoryGroup addRepository: aRepository ]! !

!WADevelopment class methodsFor: 'monticello-repositories' stamp: 'obi 10/7/2009 07:38'!
addSeasideLGPLRepository
	self addRepository: self defaultLGPLRepository toWorkingCopies: (self workingCopiesFor: self packagesLGPL)! !

!WADevelopment class methodsFor: 'monticello-repositories' stamp: 'obi 10/6/2009 17:28'!
addSeasideRepository
	self addRepository: self defaultRepository toWorkingCopies: (self workingCopiesFor: self packagesMIT)! !

!WADevelopment class methodsFor: 'accessing' stamp: 'lr 12/3/2009 10:22'!
browserEnvironment
	"Answer a browser environment of all loaded Seaside code."
	"self browserEnvironment open"
	
	| browserEnvironmentClass browserEnvironment |
	browserEnvironmentClass := (Smalltalk classNamed: #BrowserEnvironment)
		ifNil: [ self error: 'Please install the Smalltalk Refactoring Browser.' ].
	browserEnvironment := browserEnvironmentClass new 
		forPackageNames: (self packages collect: [ :each | each packageName ]).
	^ browserEnvironment label: 'Seaside Packages'! !

!WADevelopment class methodsFor: 'tools' stamp: 'jf 1/12/2009 13:03'!
cleanupCategories
	"Remove all the empty categories and fix common inconsistencies in categorie names and protocols."

	| cleanup new |
	cleanup := [ :class :protocol |
		(new := protocol asLowercase).
		(new = #'initialize-release' or: [ new = #'class-initialization' ])
			ifTrue: [ new := #initialization ].
		(new = #'instance-creation')
			ifTrue: [ new := #'instance creation' ].
		(new = protocol)
			ifFalse: [ class organization renameCategory: protocol toBe: new ].
		class organization removeEmptyCategories ].
	Smalltalk organization removeEmptyCategories.
	self packages
		do: [ :package |
			package classesAndMetaClasses do: [ :class |
				(package coreCategoriesForClass: class)
					do: [ :protocol | cleanup value: class value: protocol ] ].
			package extensionClasses do: [ :class |
				(package extensionCategoriesForClass: class)
					do: [ :protocol | cleanup value: class value: protocol ] ] ]
		displayingProgress: 'cleaning up'! !

!WADevelopment class methodsFor: 'tools' stamp: 'lr 2/28/2009 22:48'!
commentCoverage
	"Calculates comment coverage for aPackage, including method extensions. For every package it prints on the transcript:
	
		- the package name
		- number of classes
		- number of commented classes
		- percentage of commented classes
		- number of methods
		- number of commented methods
		- percentage of commented methods
		- lines of code"

	| allClasses classesCommented allMethods methodsCommented linesOfCode |
	self packages do: [ :package |
		allClasses := classesCommented := 0.
		package classes do: [ :theClass |
			allClasses := allClasses + 1.
			theClass hasComment
				ifTrue: [ classesCommented := classesCommented + 1 ] ].
		allMethods := methodsCommented := linesOfCode := 0.
		package methods do: [ :method |
			allMethods := allMethods + 1.
			linesOfCode := linesOfCode + method compiledMethod linesOfCode.
			(method actualClass firstPrecodeCommentFor: method methodSymbol) isNil
				ifFalse: [ methodsCommented := methodsCommented + 1 ] ].
		Transcript
			show: package packageName; tab;
			show: allClasses; tab;
			show: classesCommented; tab;
			show: (100.0 * classesCommented / (allClasses max: 1) truncateTo: 0.01); tab;
			show: allMethods; tab;
			show: methodsCommented; tab;
			show: (100.0 * methodsCommented / (allMethods max: 1) truncateTo: 0.01); tab;
			show: linesOfCode; cr ]! !

!WADevelopment class methodsFor: 'monticello' stamp: 'jf 11/3/2008 22:19'!
commitModifiedWorkingCopies
	self commitWorkingCopies: self modifiedWorkingCopies! !

!WADevelopment class methodsFor: 'monticello' stamp: 'jf 11/3/2008 22:18'!
commitWorkingCopies
	self commitWorkingCopies: self workingCopies! !

!WADevelopment class methodsFor: 'monticello' stamp: 'jf 8/31/2009 14:48'!
commitWorkingCopies: aCollection
	"Send changes from your working copy to the repository."

	| pair version repository |
	pair := nil.
	repository := self defaultRepository.
	(repository user isEmpty or: [ repository password isEmpty ])
		ifTrue: [ ^ self error: 'Username and password required for Seaside repository.'].
	
	aCollection
		do: [ :each | 
			each needsSaving ifTrue: [ 
				version := [ each newVersion ]
					on: MCVersionNameAndMessageRequest
					do: [ :notifcation |
						notifcation resume: (pair isNil
							ifTrue: [ pair := notifcation outer ]
							ifFalse: [ Array with: notifcation suggestedName with: pair last ]) ].
				Transcript show: version summary; cr.
				repository storeVersion: version ] ]
		displayingProgress: 'Committing'! !

!WADevelopment class methodsFor: 'monticello-repositories' stamp: 'obi 10/7/2009 07:38'!
defaultLGPLRepository
	^ self seaside30LGPLRepository! !

!WADevelopment class methodsFor: 'monticello-repositories' stamp: 'jf 9/2/2009 09:16'!
defaultRepository
	^ self seaside30Repository! !

!WADevelopment class methodsFor: 'monticello' stamp: 'jf 11/3/2008 22:19'!
diffModifiedWorkingCopies
	self diffWorkingCopies: self modifiedWorkingCopies! !

!WADevelopment class methodsFor: 'monticello' stamp: 'jf 11/3/2008 22:18'!
diffWorkingCopies
	self diffWorkingCopies: self workingCopies! !

!WADevelopment class methodsFor: 'monticello' stamp: 'jf 9/30/2009 21:46'!
diffWorkingCopies: aCollection
	"Display the differences between the working copy and the base."
	
	| patches patch |
	patches := OrderedCollection new.
	aCollection
		do: [ :each |
			patch := each package snapshot 
				patchRelativeToBase: (each ancestors isEmpty
					ifTrue: [ MCSnapshot empty ]
					ifFalse: [ (each repositoryGroup versionWithInfo: each ancestors first) snapshot ]).
			each modified: patch isEmpty not.
			patches addAll: patch operations ]
		displayingProgress: 'Diffing'.
	(MCPatch operations: patches)
		browse! !

!WADevelopment class methodsFor: 'requests' stamp: 'pmm 12/13/2008 11:25'!
ensureAuthorInitials
	"Makes sure the author initials for the image are set."
	^self ensureAuthorInitialsWith: [ :question |
		self request: question ]! !

!WADevelopment class methodsFor: 'requests' stamp: 'pmm 11/17/2009 07:05'!
ensureAuthorInitialsWith: aOneArgumentBlock
	"Makes sure the author initials for the image are set.
	aOneArgumentBlock is called with a qestion string as an argument when the initials are not set, it answers the initials"
	
	(Smalltalk version beginsWith: 'Pharo')
		ifTrue: [
			"Special Pharo code path to avoid deprecation"
			| author |
			author := Smalltalk at: #Author.
			author isNil
				ifTrue: [ ^ self error: 'unsupported Pharo version' ].
			author fullNamePerSe isEmpty
				ifTrue: [ author fullName: (aOneArgumentBlock value: 'Please enter your initials:') ] ]
		ifFalse: [
			Utilities authorInitialsPerSe isEmpty
				ifTrue: [ Utilities setAuthorInitials: (aOneArgumentBlock value: 'Please enter your initials:') ] ]! !

!WADevelopment class methodsFor: 'accessing' stamp: 'lr 12/14/2009 20:16'!
gofer
	"Answer a gopher instance."
	"self gofer update; recompile"
	
	| goferClass gofer |
	goferClass := (Smalltalk classNamed: #Gofer)
		ifNil: [ self error: 'Please install Gofer.' ].
	gofer := goferClass new.
	gofer repository: self defaultRepository.
	GRPackage packages do: [ :package |
		package isMIT 
			ifTrue: [ gofer package: package name ] ].
	^ gofer! !

!WADevelopment class methodsFor: 'tools' stamp: 'jf 9/30/2009 00:19'!
graphFunctionalDependencies: functionalBool tests: testsBool 
	| packages analyzer |
	packages := self packages.
	testsBool ifFalse: 
		[ packages := packages reject: [ :each | each packageName includesSubString: '-Tests-' ] ].
	analyzer := (Smalltalk classNamed: #PDPackageAnalyzer) onPackages: packages.
	functionalBool ifTrue: 
		[ GRPackage packages 
			do: 
				[ :package | 
				package dependencies do: 
					[ :dep | 
					| source target |
					source := analyzer relation 
						packageNamed: package name
						ifAbsent: [ nil ].
					target := analyzer relation 
						packageNamed: dep name
						ifAbsent: [ nil ].
					(source notNil and: [ target notNil ]) ifTrue: 
						[ source add: ((Smalltalk classNamed: #PDFunctionalDependency) 
								from: source
								to: target) ] ] ]
			displayingProgress: 'Adding functional dependencies' ].
	analyzer save: 'seaside.dot'! !

!WADevelopment class methodsFor: 'monticello-repositories' stamp: 'jf 8/31/2009 15:02'!
httpRepositoryWithUrlString: aString
	^ MCRepositoryGroup default repositories
		detect: [:each |
			(each isKindOf: MCHttpRepository) and:
				[ (each locationWithTrailingSlash = aString) ] ]
		ifNone: [
			"addRepository: should return its parameter but currently doesn't in
			many MC versions"
			| repository |
			repository := MCHttpRepository
				location: aString
				user: ''
				password: ''.
			MCRepositoryGroup default addRepository: repository.
			repository ]! !

!WADevelopment class methodsFor: 'monticello-accessing' stamp: 'obi 9/30/2009 22:46'!
loadedVersionNames
	"Answer all the loaded version names of Seaside. WADevelopment loadedVersionNames"
	
	^(self workingCopies select:[:each | each  ancestors notEmpty]) collect: [:each | each ancestors first name]! !

!WADevelopment class methodsFor: 'tools' stamp: 'jf 9/30/2009 01:10'!
methodsOf: aString 
	"Lists all the methods with author initials aString."
	| methods |
	methods := self packages gather: 
		[ :package | 
		package methods select: [ :method | method timeStamp beginsWith: aString ] ].
	MessageSet 
		openMessageList: methods
		name: methods size greaseString , ' methods of ' , aString! !

!WADevelopment class methodsFor: 'monticello-accessing' stamp: 'lr 11/3/2008 23:05'!
modifiedWorkingCopies
	"Answer all the modified working copies of Seaside."
	
	^ self workingCopies select: [ :each | each modified ]! !

!WADevelopment class methodsFor: 'accessing' stamp: 'jf 2/6/2010 04:34'!
packages
	"Answer a list of Seaside related packages as PackageInfo instances."
	^ (GRPackage packages sorted: [ :a :b | a name < b name ]) collect: [ :each | PackageInfo named: each name ]! !

!WADevelopment class methodsFor: 'accessing' stamp: 'jf 2/6/2010 04:34'!
packagesLGPL
	"Answer a list of Seaside LGPL licensed packages as PackageInfo instances"
	^ ((GRPackage packages select: [:each | each isLGPL ]) sorted: [ :a :b | a name < b name ]) collect: [ :each | PackageInfo named: each name ]! !

!WADevelopment class methodsFor: 'accessing' stamp: 'jf 2/6/2010 04:34'!
packagesMIT
	"Answer a list of Seaside MIT licensed packages as PackageInfo instances."
	^ ((GRPackage packages select: [:each | each isMIT ]) sorted: [ :a :b | a name < b name ]) collect: [ :each | PackageInfo named: each name ]! !

!WADevelopment class methodsFor: 'tools' stamp: 'lr 1/11/2009 21:13'!
recompile
	"Recompiles the all Seaside related packages."
	
	self packages
		do: [ :package |
			package methods
				do: [ :method | method actualClass recompile: method methodSymbol ]
				displayingProgress: package packageName ]
		displayingProgress: 'Recompiling'! !

!WADevelopment class methodsFor: 'monticello-repositories' stamp: 'jf 9/2/2009 09:24'!
removeRepository: aRepository fromWorkingCopies: aCollection
	aCollection do: [ :each | each repositoryGroup removeRepository: aRepository ]! !

!WADevelopment class methodsFor: 'tools' stamp: 'lr 8/30/2009 17:29'!
renamePackage: oldPackageName to: newPackageName
	"self renamePackage: 'Seaside-Squeak-Development-Core' to: 'Seaside-Pharo-Development-Core'"
	
	| oldPackage newPackage oldWorkingCopy newWorkingCopy |
	oldPackage := PackageInfo named: oldPackageName.
	newPackage := PackageInfo named: newPackageName.
	
	" rename system categories "
	oldPackage systemCategories do: [ :oldCategory |
		| newCategory |
		newCategory := oldCategory allButFirst: oldPackage systemCategoryPrefix size.
		Smalltalk organization renameCategory: oldCategory toBe: newPackage systemCategoryPrefix , newCategory ].
	
	" rename method categories "
	oldPackage extensionClasses do: [ :extensionClass |
		(oldPackage extensionCategoriesForClass: extensionClass) do: [ :oldProtocol |
			| newProtocol |
			newProtocol := oldProtocol allButFirst: oldPackage methodCategoryPrefix size.
			extensionClass organization renameCategory: oldProtocol toBe: newPackage methodCategoryPrefix , newProtocol ] ].
	
	" update monticello packages "
	oldWorkingCopy := MCWorkingCopy forPackage: (MCPackage named: oldPackageName).
	newWorkingCopy := MCWorkingCopy forPackage: (MCPackage named: newPackageName).
	newWorkingCopy repositoryGroup: oldWorkingCopy repositoryGroup; modified: true.
	
	" test is all methos have been caught "
	oldPackage methods isEmpty
		ifTrue: [ oldWorkingCopy unload. PackageOrganizer default unregisterPackage: oldPackage ]
		ifFalse: [ self error: 'Some code entities remain in the old package, please migrate manually.' ]! !

!WADevelopment class methodsFor: 'requests' stamp: 'lr 10/18/2008 09:22'!
request: aString
	^self requestor request: aString! !

!WADevelopment class methodsFor: 'private' stamp: 'lr 10/18/2008 09:23'!
requestor
	^((Smalltalk hasClassNamed: #UIManager)
		ifTrue: [ (Smalltalk classNamed: #UIManager) default ]
		ifFalse: [
			"can be the case for kernel image"
			Smalltalk classNamed: #FillInTheBlank ])! !

!WADevelopment class methodsFor: 'monticello' stamp: 'jf 11/3/2008 22:19'!
revertModifiedWorkingCopies
	self revertWorkingCopies: self modifiedWorkingCopies! !

!WADevelopment class methodsFor: 'monticello' stamp: 'jf 11/3/2008 22:18'!
revertWorkingCopies
	self revertWorkingCopies: self workingCopies! !

!WADevelopment class methodsFor: 'monticello' stamp: 'jf 11/3/2008 22:17'!
revertWorkingCopies: aCollection
	"Restore pristine working copies."
	
	aCollection
		do: [ :each | 
			each modified: false.
			(each repositoryGroup versionWithInfo: each ancestors first)
				load ]
		displayingProgress: 'Reverting'! !

!WADevelopment class methodsFor: 'monticello-repositories' stamp: 'jf 9/2/2009 09:15'!
seaside30LGPLRepository
	^ self httpRepositoryWithUrlString: 'http://www.squeaksource.com/Seaside30LGPL/'! !

!WADevelopment class methodsFor: 'monticello-repositories' stamp: 'jf 9/2/2009 09:15'!
seaside30Repository
	^ self httpRepositoryWithUrlString: 'http://www.squeaksource.com/Seaside30/'! !

!WADevelopment class methodsFor: 'monticello' stamp: 'jf 11/3/2008 22:17'!
updateWorkingCopies
	self updateWorkingCopies: self workingCopies! !

!WADevelopment class methodsFor: 'monticello' stamp: 'jf 6/4/2009 10:43'!
updateWorkingCopies: aCollection
	"Bring the latest changes from the repository into the image."
	
	| versions loaded version |
	aCollection
		do: [ :each |
			versions := each possiblyNewerVersions.
			versions isEmpty ifFalse: [
				loaded := false.
				each repositoryGroup repositories do: [ :repository | 
					loaded ifFalse: [
						version := [ repository loadVersionFromFileNamed: versions first , '.mcz' ]
							on: FileDoesNotExistException, FileStreamException
							do: [ :err | nil ].
						version isNil ifFalse: [ version load. loaded := true ] ] ] ] ]
		displayingProgress: 'Updating'! !

!WADevelopment class methodsFor: 'monticello-accessing' stamp: 'obi 10/6/2009 17:27'!
workingCopies
	"Answer all the loaded working copies of Seaside."
	^self workingCopiesFor: self packages.
! !

!WADevelopment class methodsFor: 'monticello-accessing' stamp: 'obi 10/6/2009 17:27'!
workingCopiesFor: packages
	"Answer all the loaded working copies of Seaside."
	^ (MCWorkingCopy registry
		select: [ :each | packages includes: each packageInfo ])
			asSortedCollection: [ :a :b | a packageName < b packageName ]! !

WAObject subclass: #WAMemoryItem
	instanceVariableNames: 'class count size'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Pharo-Development-Core'!

!WAMemoryItem class methodsFor: 'instance creation' stamp: 'lr 2/20/2009 16:00'!
on: aClass
	^ self basicNew initializeOn: aClass! !

!WAMemoryItem methodsFor: 'public' stamp: 'lr 2/20/2009 15:55'!
accumulate: anObject
	count := count + 1.
	size := size + (self sizeOfObject: anObject)! !

!WAMemoryItem methodsFor: 'accessing' stamp: 'lr 2/20/2009 16:02'!
count
	^ count! !

!WAMemoryItem methodsFor: 'initialization' stamp: 'lr 2/20/2009 16:00'!
initializeOn: aClass
	self initialize.
	class := aClass.
	count := size := 0! !

!WAMemoryItem methodsFor: 'accessing' stamp: 'lr 2/20/2009 16:02'!
name
	^ class name! !

!WAMemoryItem methodsFor: 'accessing' stamp: 'lr 2/20/2009 16:02'!
size
	^ size! !

!WAMemoryItem methodsFor: 'private' stamp: 'lr 2/20/2009 16:01'!
sizeOfObject: anObject
	| headerSize instanceSize variableSize |
	headerSize := anObject class indexIfCompact > 0
		ifTrue: [ 4 ]
		ifFalse: [ 8 ].
	instanceSize := anObject class instSize.
	variableSize := anObject class isBytes
		ifTrue: [ anObject basicSize ]
		ifFalse: [
			anObject class isWords
				ifTrue: [ Smalltalk wordSize * anObject basicSize ]
				ifFalse: [ Smalltalk wordSize * anObject basicSize // 2 ] ].
	^ headerSize + instanceSize + variableSize! !

WAObject subclass: #WAProfilerItem
	instanceVariableNames: 'start tally request'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Pharo-Development-Core-Profiler'!

!WAProfilerItem class methodsFor: 'instance creation' stamp: 'jf 3/13/2009 18:14'!
request: aRequest
	^ self basicNew initializeOn: aRequest; yourself! !

!WAProfilerItem methodsFor: 'accessing-dynamic' stamp: 'lr 12/30/2008 16:34'!
duration
	"Answer the receiver's run time."

	^ self tally time milliSeconds! !

!WAProfilerItem methodsFor: 'initialization' stamp: 'jf 3/13/2009 18:14'!
initializeOn: aRequest
	self initialize.
	request := aRequest! !

!WAProfilerItem methodsFor: 'accessing-dynamic' stamp: 'pmm 11/21/2009 12:55'!
leavesBySends
	^ String streamContents: [ :stream | 
		self tally leavesPrintExactOn: stream ]! !

!WAProfilerItem methodsFor: 'accessing-dynamic' stamp: 'pmm 11/21/2009 12:56'!
leavesByTime
	^ String streamContents: [ :stream | 
		self tally leavesPrintOn: stream threshold: 1 ]! !

!WAProfilerItem methodsFor: 'public' stamp: 'pmm 11/21/2009 12:47'!
profile: aBlock
	start := TimeStamp now.
	tally := MessageTally new.
	tally reportOtherProcesses: false.
	^ tally spyEvery: 1 on: aBlock! !

!WAProfilerItem methodsFor: 'accessing' stamp: 'lr 12/30/2008 16:37'!
request
	"Answer the request causing this profile."
	
	^ request! !

!WAProfilerItem methodsFor: 'accessing' stamp: 'lr 12/30/2008 16:17'!
start
	"Answer the start timestamp when this profile was collected."

	^ start! !

!WAProfilerItem methodsFor: 'accessing-dynamic' stamp: 'lr 12/30/2008 16:58'!
stats
	^ String streamContents: [ :stream | 
		self tally reportGCStatsOn: stream ]! !

!WAProfilerItem methodsFor: 'accessing' stamp: 'lr 12/30/2008 16:13'!
tally
	"Answer the tally of this profile."

	^ tally! !

!WAProfilerItem methodsFor: 'accessing-dynamic' stamp: 'lr 12/30/2008 16:59'!
tree
	^ String streamContents: [ :stream | 
		self tally
			maxTabs: SmallInteger maxVal;
			maxClassNameSize: SmallInteger maxVal;
			maxClassPlusSelectorSize: SmallInteger maxVal;
			treePrintOn: stream 
			tabs: OrderedCollection new
			thisTab: String new
			total: tally tally
			totalTime: tally time
			tallyExact: true
			orThreshold: 2 ]! !

WAWalkback subclass: #WASqueakWalkback
	instanceVariableNames: 'frames'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Pharo-Development-Core'!

!WASqueakWalkback commentStamp: 'jf 2/21/2009 00:00' prior: 0!
A Squeak-specific implementation of WAWalkback.!

!WASqueakWalkback class methodsFor: 'class initialization' stamp: 'jf 2/20/2009 23:29'!
initialize
	self select! !

!WASqueakWalkback class methodsFor: 'class initialization' stamp: 'jf 2/20/2009 23:29'!
unload
	self unselect! !

!WASqueakWalkback methodsFor: 'accessing' stamp: 'jf 2/20/2009 23:16'!
frameCount
	^ frames size! !

!WASqueakWalkback methodsFor: 'accessing' stamp: 'jf 2/20/2009 23:08'!
frames
	^ frames first: (self limit min: frames size)! !

!WASqueakWalkback methodsFor: 'initialization' stamp: 'jf 2/20/2009 23:02'!
initializeWithException: anException
	| context |
	super initializeWithException: anException.
	context := anException signalerContext.
	frames := OrderedCollection new.
	[ context isNil ] whileFalse: [
		frames add: context.
		context := context sender ]! !

!WASqueakWalkback methodsFor: 'rendering-stack' stamp: 'lr 9/29/2009 22:47'!
renderObject: anObject labelled: aString on: html
	html definitionTerm: [
		html anchor
			callback: [ (WAInspectorHaloPlugin on: anObject for: self) execute ];
			with: aString ].
	html definitionData: (self safePrintString: anObject)! !

!WASqueakWalkback methodsFor: 'rendering-stack' stamp: 'lr 10/7/2009 23:08'!
renderStackFrame: aContext on: html
	html definitionList: [
		self 
			renderObject: aContext
			labelled: 'thisContext'
			on: html.
		self 
			renderObject: aContext receiver
			labelled: 'self'
			on: html.
		self 
			tempNamesAndValuesIn: aContext 
			do: [ :name :value |
				self 
					renderObject: value
					labelled: name
					on: html ] ]! !

!WASqueakWalkback methodsFor: 'rendering' stamp: 'jf 2/20/2009 23:11'!
renderStackOn: html
	html heading level: 3; with: 'Stack Trace'.
	html orderedList: [
		self frames do: [ :each |
			html listItem: [
				self renderStackFrame: each on: html ] ] ]! !

!WASqueakWalkback methodsFor: 'private' stamp: 'jf 2/20/2009 23:55'!
safePrintString: anObject
	^ [ anObject printStringLimitedTo: 100 ]
		on: Error
		do: [ :err | 'unprintable ' , anObject class name ]! !

!WASqueakWalkback methodsFor: 'private' stamp: 'lr 6/28/2009 23:09'!
tempNamesAndValuesIn: aContext do: aTwoArgumentBlock
	"Evaluate aTwoArgumentBlock for every temp in aContext with the name of the temp and the current value. The code is supposed to work on Squeak and Pharo closure and non-closure images."

	| tempNames |
	tempNames := aContext tempNames.
	(aContext respondsTo: #namedTempAt:)
		ifTrue: [
			tempNames keysAndValuesDo: [ :index :each |
				aTwoArgumentBlock
					value: each
					value: (aContext namedTempAt: index) ] ]
		ifFalse: [
			tempNames do: [ :each |
				aTwoArgumentBlock
					value: each
					value: (aContext tempAt: (tempNames indexOf: each)) ] ]! !

!Collection methodsFor: '*seaside-pharo-development-core' stamp: 'lr 5/9/2007 08:47'!
inspectorFields
	| i |
	i := 0.
	^ self asArray collect: [ :each | i := i + 1. i -> each ]! !

WATool subclass: #WABrowser
	instanceVariableNames: 'contents model message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Pharo-Development-Core'!

!WABrowser class methodsFor: 'accessing' stamp: 'pmm 4/1/2008 07:05'!
applicationName
	"needed for RSRSS"
	^'tools/classbrowser'! !

!WABrowser class methodsFor: 'accessing' stamp: 'lr 1/8/2008 20:14'!
canBeRoot
	^ true! !

!WABrowser class methodsFor: 'accessing' stamp: 'lr 2/12/2007 23:18'!
description
	^ 'Smalltalk Code Browser'! !

!WABrowser class methodsFor: 'instance creation' stamp: 'lr 11/3/2006 13:50'!
fullOnClass: aClass
	^ self fullOnClass: aClass selector: nil! !

!WABrowser class methodsFor: 'instance creation' stamp: 'lr 11/3/2006 13:50'!
fullOnClass: aClass selector: aSelector
	^ self new model: (Browser new setClass: aClass selector: aSelector)! !

!WABrowser class methodsFor: 'initialization' stamp: 'jf 10/26/2008 17:44'!
initialize
	WAAdmin register: self asApplicationAt: self applicationName! !

!WABrowser methodsFor: 'actions' stamp: 'pmm 12/13/2008 11:25'!
accept
	| save |
	WADevelopment ensureAuthorInitialsWith: [ :question |
		self request: question ].
	message := nil.
	save := contents.
	model contents: contents notifying: self.
	contents := save! !

!WABrowser methodsFor: 'accessing' stamp: 'lr 8/1/2007 11:15'!
columns
	^ #( systemCategory class messageCategory message )! !

!WABrowser methodsFor: 'accessing' stamp: 'ab 10/10/2002 23:50'!
contents
	^contents! !

!WABrowser methodsFor: 'accessing' stamp: 'jf 9/30/2009 00:19'!
contents: aString 
	"the browser sends CRLF as line ending but Squeak wants CR"
	contents := GRPlatform current convertToSmalltalkNewlines: aString! !

!WABrowser methodsFor: 'processing' stamp: 'pmm 8/31/2008 18:11'!
initialRequest: aRequest
	| theClass |
	super initialRequest: aRequest.
	aRequest fields at: 'class' ifPresent: [ :className |
		Smalltalk at: className asSymbol ifPresent: [ :class |
			theClass := (aRequest fields includesKey: 'meta')
				ifTrue: [ class class ]
				ifFalse: [ class ].
			self model setClass: theClass selector: nil.
			aRequest fields at: 'method' ifPresent: [ :selector |
				self model setClass: theClass selector: selector asSymbol ] ] ]! !

!WABrowser methodsFor: 'initialization' stamp: 'lr 5/28/2007 18:09'!
initialize
	super initialize.
	self model: Browser new! !

!WABrowser methodsFor: 'accessing' stamp: 'lr 10/28/2007 14:42'!
model
	^model! !

!WABrowser methodsFor: 'accessing' stamp: 'jf 3/14/2009 01:23'!
model: aBrowserModel
	model notNil ifTrue: [ model removeDependent: self ].
	aBrowserModel notNil ifTrue: [ aBrowserModel addDependent: self ].
	model := aBrowserModel.
	contents := model contents! !

!WABrowser methodsFor: 'notifying' stamp: 'ab 10/10/2002 23:36'!
notify: aString at: location in: sourceStream
	message := aString allButLast: 3! !

!WABrowser methodsFor: 'rendering' stamp: 'lr 12/27/2008 17:16'!
renderColumnsOn: html
	html div class: 'columns'; with: [
		self columns do: [ :each |
			html form class: 'column'; with: [
				| array |
				array := self model perform: (each , 'List') asSymbol.
				html div: [
					html select
						size: 10;
						list: array;
						onChange: 'submit()';
						selected: (array
							at: (self model perform: (each , 'ListIndex') asSymbol)
							ifAbsent: [ nil ]);
						callback: [ :item |
							self model
								perform: (each , 'ListIndex:') asSymbol
								with: (array indexOf: item) ] ] ] ] ] ! !

!WABrowser methodsFor: 'rendering' stamp: 'lr 4/9/2008 10:58'!
renderContentOn: html
	self
		renderColumnsOn: html;
		renderModeButtonsOn: html;
		renderMessageOn: html;
		renderContentPaneOn: html! !

!WABrowser methodsFor: 'rendering' stamp: 'pmm 8/31/2008 15:20'!
renderContentPaneOn: html
	html form class: 'definition'; with: [
	 	html div: [
			html textArea on: #contents of: self.
			html break.
			html submitButton on: #accept of: self ] ]! !

!WABrowser methodsFor: 'rendering' stamp: 'lr 2/28/2009 22:40'!
renderMessageOn: html
	message isNil
		ifFalse: [ html strong: message ]! !

!WABrowser methodsFor: 'rendering' stamp: 'pmm 8/31/2008 15:20'!
renderModeButtonsOn: html
	html form class: 'mode'; with: [
		html div: [
			html submitButton
				callback: [ self showInstance ];
				value: 'instance'.
			html submitButton
				callback: [ self showHelp ];
				value: 'help'.
			html submitButton
				callback: [ self showClass ];
				value: 'class' ] ]! !

!WABrowser methodsFor: 'notifying' stamp: 'ab 10/11/2002 00:00'!
select! !

!WABrowser methodsFor: 'actions' stamp: 'lr 10/28/2007 14:42'!
selectFrom: aPos to: anotherPos! !

!WABrowser methodsFor: 'accessing' stamp: 'ab 10/10/2002 23:44'!
selectionInterval
	^ 1 to: 1! !

!WABrowser methodsFor: 'actions' stamp: 'LR 9/13/2002 20:38'!
showClass
	self model indicateClassMessages! !

!WABrowser methodsFor: 'actions' stamp: 'LR 9/13/2002 20:42'!
showHelp
	self model plusButtonHit! !

!WABrowser methodsFor: 'actions' stamp: 'LR 9/13/2002 20:39'!
showInstance
	self model indicateInstanceMessages! !

!WABrowser methodsFor: 'accessing' stamp: 'lr 4/15/2007 19:55'!
states
	^ Array with: model! !

!WABrowser methodsFor: 'rendering' stamp: 'lr 8/6/2007 09:51'!
style
	^ 'form, div.columns {
	padding: 2px;
	margin: 0;
}

form.column {
	padding: 0;
	width: 25%;
	float: left;
}
form.column select {
	width: 100%;
	border: 1px solid #aaa;
}

form.mode {
	clear: both;
}

form.definition textarea {
	width: 100%;
	height: 200px;
	border: 1px solid #aaa;
}'! !

!WABrowser methodsFor: 'accessing' stamp: 'ab 10/10/2002 23:44'!
text
	^ Text fromString: ''! !

!WABrowser methodsFor: 'notifying' stamp: 'ab 10/10/2002 01:45'!
update: aSymbol
	contents := model contents! !

!WABrowser methodsFor: 'processing' stamp: 'jf 2/15/2010 02:05'!
updateUrl: aUrl
	super updateUrl: aUrl.
	self model selectedClass isNil ifFalse: [
		aUrl addField: 'class' value: self model selectedClass fullName.
		self model metaClassIndicated
			ifTrue: [ aUrl addField: 'meta' ] ].
	self model selectedMessageName isNil 
		ifFalse: [ aUrl addField: 'method' value: self model selectedMessageName ]! !

WATool subclass: #WAMemory
	instanceVariableNames: 'instances table formatter'
	classVariableNames: 'IgnoredClasses'
	poolDictionaries: ''
	category: 'Seaside-Pharo-Development-Core'!

!WAMemory class methodsFor: 'initialization' stamp: 'jf 2/21/2009 02:47'!
initialize
	"We don't want to include our own data in the mix.
	Process is excluded because otherwise our Semaphores pull them in and the Process is
	obviously not held onto by the Semaphore indefinitely.
	CompiledMethods are basically always going to be around in the image anyway so are
	not really session-specific data."
	IgnoredClasses := Array with: WAMemory with: WAMemoryItem with: Process with: CompiledMethod! !

!WAMemory methodsFor: 'private' stamp: 'jf 2/21/2009 01:45'!
accumulate: anObject
	(instances at: anObject class ifAbsentPut: [ WAMemoryItem on: anObject class ])
		accumulate: anObject! !

!WAMemory methodsFor: 'private' stamp: 'lr 2/20/2009 22:13'!
buildTable
	^ WATableReport new
		rows: instances values;
		columns: (Array
			with: (WAReportColumn new
				title: 'Class';
				selector: #name;
				sortBlock: [ :a :b | a < b ];
				yourself)
			with: (WAReportColumn new
				title: 'Instances';
				selector: #count;
				sortBlock: [ :a :b | a > b ];
				cssClass: 'right';
				hasTotal: true;
				yourself)
			with: (WAReportColumn new
				title: 'Total Size';
				selector: #size;
				sortBlock: [ :a :b | a > b ];
				formatBlock: [ :each | formatter print: each ];
				cssClass: 'right';
				hasTotal: true;
				yourself));
		yourself! !

!WAMemory methodsFor: 'accessing' stamp: 'avi 2/10/2004 16:15'!
children
	^ Array with: table! !

!WAMemory methodsFor: 'initialization' stamp: 'jf 9/30/2009 00:32'!
initialize
	super initialize.
	instances := IdentityDictionary new.
	formatter := GRPrinter binaryFileSize.
	self session 
		traverseWithMemory: self
		seen: (IdentitySet with: self session).
	table := self buildTable! !

!WAMemory methodsFor: 'rendering' stamp: 'lr 2/20/2009 16:30'!
renderContentOn: html
	html div
		class: 'tool';
		with: table! !

!WAMemory methodsFor: 'accessing' stamp: 'jf 12/29/2009 20:42'!
totalInstances
	^ instances inject: 0 into: [ :sum :each | sum + each count ]! !

!WAMemory methodsFor: 'accessing' stamp: 'jf 12/29/2009 20:42'!
totalSize
	^ instances inject: 0 into: [ :sum :each | sum + each size ]! !

!WAMemory methodsFor: 'private' stamp: 'jf 2/21/2009 02:34'!
traverse: anObject value: aValue seen: anIdentitySet
	(aValue isNil or: [ aValue isLiteral or: [ aValue isBehavior or: [ (anIdentitySet includes: aValue) or: [ IgnoredClasses anySatisfy: [ :each | aValue isKindOf: each ] ] ] ] ])
		ifTrue: [ ^ self ].
	aValue traverseWithMemory: self seen: anIdentitySet! !

WATool subclass: #WAProfilerTool
	instanceVariableNames: 'report items message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Pharo-Development-Core-Profiler'!

!WAProfilerTool class methodsFor: 'instance creation' stamp: 'jf 3/13/2009 18:15'!
on: aCollection
	^ self basicNew initializeOn: aCollection; yourself! !

!WAProfilerTool methodsFor: 'private' stamp: 'jf 9/30/2009 01:10'!
buildReport
	^ WATableReport new
		rows: self items;
		columns: (OrderedCollection new
				add: (WAReportColumn new
						title: 'Start';
						selector: #start;
						cssClass: 'left';
						yourself);
				add: (WAReportColumn new
						title: 'Duration';
						selector: #duration;
						cssClass: 'left';
						yourself);
				add: (WAReportColumn new
						title: 'Request';
						valueBlock: [ :value | value request method , ' ' , value request url greaseString ];
						clickBlock: [ :value | self show: (WAInspector current on: value request) ];
						yourself);
				add: (WAReportColumn new
						sortBlock: nil;
						title: 'Commands';
						valueBlock: 
							[ :value :html | 
							self 
								renderCommands: value
								on: html ];
						yourself);
				yourself);
		yourself! !

!WAProfilerTool methodsFor: 'accessing' stamp: 'lr 12/30/2008 16:31'!
children
	^ Array with: self report! !

!WAProfilerTool methodsFor: 'initialization' stamp: 'jf 3/13/2009 18:15'!
initializeOn: aCollection
	self initialize.
	items := aCollection! !

!WAProfilerTool methodsFor: 'accessing' stamp: 'lr 12/30/2008 16:33'!
items
	^ items! !

!WAProfilerTool methodsFor: 'accessing' stamp: 'lr 12/30/2008 16:47'!
message
	^ message! !

!WAProfilerTool methodsFor: 'rendering' stamp: 'pmm 11/21/2009 12:56'!
renderCommands: anItem on: html
	html anchor
		callback: [ message := anItem tree ];
		with: 'Tree'.
	html text: ' '.
	html anchor
		callback: [ message := anItem leavesByTime ];
		with: 'Leaves (time)'.
		html text: ' '.
	html anchor
		callback: [ message := anItem leavesBySends ];
		with: 'Leaves (sends)'.
	html text: ' '.
	html anchor
		callback: [ message := anItem stats ];
		with: 'Stats'
! !

!WAProfilerTool methodsFor: 'rendering' stamp: 'lr 2/20/2009 22:32'!
renderContentOn: html
	html div class: 'tool'; with: self report.
	self message isNil 
		ifFalse: [ html preformatted: self message ]! !

!WAProfilerTool methodsFor: 'accessing' stamp: 'lr 12/30/2008 16:30'!
report
	^ report ifNil: [ report := self buildReport ]! !

WAToolPlugin subclass: #WAMemoryToolPlugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Pharo-Development-Core-Plugins'!

!WAMemoryToolPlugin commentStamp: 'pmm 5/27/2007 18:32' prior: 0!
I display the memory usage tool.!

!WAMemoryToolPlugin methodsFor: 'actions' stamp: 'lr 9/25/2008 18:12'!
execute
	self open: WAMemory new! !

!WAMemoryToolPlugin methodsFor: 'accessing' stamp: 'lr 5/11/2007 00:24'!
label
	^ 'Memory'! !

!WAMemoryToolPlugin methodsFor: 'accessing' stamp: 'lr 5/11/2007 10:56'!
priority
	^ 500! !

!WAMemoryToolPlugin methodsFor: 'accessing' stamp: 'jf 2/21/2009 02:38'!
title
	^ 'Session Memory Use'! !

WAToolPlugin subclass: #WAProfilerToolPlugin
	instanceVariableNames: 'filter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Pharo-Development-Core-Profiler'!

!WAProfilerToolPlugin commentStamp: 'pmm 5/27/2007 18:32' prior: 0!
I display the rendering profiler.!

!WAProfilerToolPlugin methodsFor: 'actions' stamp: 'lr 12/30/2008 17:02'!
execute
	self isProfiling
		ifFalse: [ self session addFilter: self filter reset ]
		ifTrue: [
			self session removeFilter: self filter.
			self open: (WAProfilerTool on: self filter items) ]! !

!WAProfilerToolPlugin methodsFor: 'accessing' stamp: 'lr 12/30/2008 16:08'!
filter
	^ filter ifNil: [ filter := WAProfilerFilter new ]! !

!WAProfilerToolPlugin methodsFor: 'testing' stamp: 'lr 12/30/2008 16:08'!
isProfiling
	^ self session filters includes: self filter! !

!WAProfilerToolPlugin methodsFor: 'accessing' stamp: 'lr 12/30/2008 16:22'!
label
	^ self isProfiling
		ifFalse: [ 'Profile' ]
		ifTrue: [ 'Stop Profile' ]! !

!WAProfilerToolPlugin methodsFor: 'accessing' stamp: 'lr 5/11/2007 10:56'!
priority
	^ 400! !

WAInspector subclass: #WASqueakInspector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Pharo-Development-Core'!

!WASqueakInspector commentStamp: 'jf 3/3/2009 22:39' prior: 0!
A Squeak-specific implementation of a web-based object inspector.!

!WASqueakInspector class methodsFor: 'class initialization' stamp: 'jf 3/3/2009 22:06'!
initialize
	self select! !

!WASqueakInspector class methodsFor: 'class initialization' stamp: 'jf 3/3/2009 22:06'!
unload
	self unselect! !

!WASqueakInspector methodsFor: 'actions' stamp: 'jf 3/3/2009 22:09'!
doIt
	| result |
	result := super doIt.
	selected ifNil: [ ^ result ].
	^ selected isNumber
		ifTrue: [ self object at: selected put: result ]
		ifFalse: [ self object instVarNamed: selected put: result ]! !

!WASqueakInspector methodsFor: 'actions' stamp: 'jf 3/3/2009 21:50'!
evaluate
	^ self object class evaluatorClass new
		evaluate: input
		in: thisContext
		to: self object! !

!WASqueakInspector methodsFor: 'private' stamp: 'jf 3/3/2009 21:48'!
fieldsFor: anObject
	^ anObject inspectorFields! !

!Object methodsFor: '*seaside-pharo-development-core' stamp: 'lr 2/6/2010 11:52'!
inspectorFields
	| members |
	members := WriteStream on: Array new.
	self class allInstVarNames 
		keysAndValuesDo: [ :index :each | members nextPut: each -> (self instVarAt: index) ].
	self class isVariable
		ifTrue: [ 1 to: self basicSize do: [ :index | members nextPut: index -> (self basicAt: index) ] ].
	^ members contents! !

!Object methodsFor: '*seaside-pharo-development-core' stamp: 'jf 2/21/2009 01:57'!
traversableIndexableVarIndexes
	^ 1 to: self basicSize! !

!Object methodsFor: '*seaside-pharo-development-core' stamp: 'jf 2/21/2009 01:55'!
traversableInstVarIndexes
	^ 1 to: self class instSize! !

!Object methodsFor: '*seaside-pharo-development-core' stamp: 'jf 2/21/2009 01:56'!
traverseWithMemory: aMemory seen: anIdentitySet
	aMemory accumulate: self.
	anIdentitySet add: self.
	self traversableInstVarIndexes do: [ :index |
		aMemory 
			traverse: self
			value: (self instVarAt: index)
			seen: anIdentitySet ].
	self traversableIndexableVarIndexes do: [ :index |
		aMemory 
			traverse: self 
			value: (self basicAt: index) 
			seen: anIdentitySet ]! !

!GRPackage class methodsFor: '*seaside-pharo-development' stamp: 'jf 8/31/2009 14:29'!
seasidePharoDevelopment
	^ self new
		name: 'Seaside-Pharo-Development';
		addDependency: 'Seaside-Development';
		addDependency: 'Seaside-Pharo-Tools-Web';
		addDependency: 'Seaside-Pharo-Flow';
		yourself	"For WAWalkbackErrorHandler"! !

!InstructionStream methodsFor: '*seaside-pharo-development' stamp: 'jf 2/21/2009 02:30'!
traversableInstVarIndexes
	"sender will always get niled out when the stack unwound so won't be captured but
	this will not have happened yet for contexts involved in the current request."
	^ super traversableInstVarIndexes copyWithout:
		(self class allInstVarNames indexOf: 'sender')! !

WAHaloPlugin subclass: #WABrowserHaloPlugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Pharo-Development-Core-Plugins'!

!WABrowserHaloPlugin commentStamp: 'pmm 5/27/2007 17:34' prior: 0!
I open a browser on the component class.!

!WABrowserHaloPlugin methodsFor: 'actions' stamp: 'dkh 11/25/2009 14:16'!
execute
	self open: (WABrowser fullOnClass: self root class)! !

!WABrowserHaloPlugin methodsFor: 'accessing' stamp: 'lr 5/11/2007 00:54'!
label
	^ 'Browser'! !

!WABrowserHaloPlugin methodsFor: 'accessing' stamp: 'lr 5/11/2007 11:02'!
priority
	^ 500! !

!WABrowserHaloPlugin methodsFor: 'accessing' stamp: 'pmm 5/16/2007 16:24'!
tooltip
	^ 'Class Browser'! !

!Dictionary methodsFor: '*seaside-pharo-development-core' stamp: 'jf 2/6/2010 04:31'!
inspectorFields
	^ self associations! !

WARequestFilter subclass: #WAProfilerFilter
	instanceVariableNames: 'items'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Pharo-Development-Core-Profiler'!

!WAProfilerFilter methodsFor: 'processing' stamp: 'lr 2/20/2009 22:23'!
handleFiltered: aRequestContext
	| item |
	item := WAProfilerItem request: aRequestContext request copy.
	[ item profile: [ super handleFiltered: aRequestContext ] ]
		ensure: [ items := items copyWith: item ]! !

!WAProfilerFilter methodsFor: 'accessing' stamp: 'lr 12/30/2008 16:00'!
items
	^ items! !

!WAProfilerFilter methodsFor: 'actions' stamp: 'lr 12/30/2008 16:23'!
reset
	items := Array new! !
WASqueakWalkback initialize!
WABrowser initialize!
WAMemory initialize!
WASqueakInspector initialize!
